BASE     := $(dir $(lastword $(MAKEFILE_LIST)))

VIEWER   ?= okular
LATEX    ?= pdflatex
BIBTEX   ?= bibtex

BUILD    ?= ./build
DOC      ?= ./doc
LOG      ?= ./log

BUILDFIG  := ${BUILD}/figure
BUILDTAB  := ${BUILD}/table
BUILDCODE := ${BUILD}/code


FIG_DIRS += ${BASE}
CONTENT_DIRS += ${BASE}

DOCUMENT       :=${DOC}/${TARGET}.pdf
DIRS           :=${BUILD} ${BUILDFIG} ${BUILDTAB} ${BUILDCODE} ${DOC} ${LOG}
GARBAGE        :=${DIRS}
DOCUMENT_LOG   :=${LOG}/${TARGET}.log
STARTFILE      := ${BASE}/main.tex

LATEXOPTIONS := -halt-on-error -interaction=nonstopmode -shell-escape -file-line-error --output-directory ${BUILD}
empty        :=
space        :=${empty} ${empty}
TEXPATHS     :=$(subst ${space},:,$(strip ${CONTENT_DIRS} ${FIG_DIRS} ${TAB_DIRS} ${CODE_DIRS}))
BIBPATHS     :=$(subst ${space},:,$(strip ${BIB_DIRS}))

LATEX        :=TEXINPUTS+=:${TEXPATHS} ${LATEX}
BIBTEX       :=BIBINPUTS+=:${BIBPATHS} ${BIBTEX}


CONTENT          := $(foreach dir,${CONTENT_DIRS},$(wildcard ${dir}/*.tex))
TEMPLATES        := $(wildcard ${BASE}/*.tex)
BIBS             := $(foreach dir, ${BIB_DIRS}   ,$(wildcard ${dir}/*.bib))

SOURCES         :=${TEMPLATES} ${CONTENT} ${TIKZ_FIGURES} ${TABLES} ${CODE} ${METAUML_TARGETS} ${BIBS}

TARGET_AUX     :=${BUILD}/$(basename $(notdir ${STARTFILE}))
TARGET_PDF     :=${BUILD}/$(basename $(notdir ${STARTFILE})).pdf

LATEX_CMD1         := echo "Running PDFLatex 1." ; ${LATEX} ${LATEXOPTIONS} -draftmode ${STARTFILE} > ${LOG}/run1.log

	BIBTEX_CMD     := echo "Running BibTex" ; ${BIBTEX} ${TARGET_AUX} > ${LOG}/bib.log || echo "Bibtex failed"
	LATEX_CMD2     := echo "Running PDFLatex 2." ; ${LATEX} ${LATEXOPTIONS} -draftmode ${STARTFILE} > ${LOG}/run2.log
	LATEX_CMD3     := echo "Running PDFLatex 3." ; ${LATEX} ${LATEXOPTIONS} ${STARTFILE} > ${LOG}/run3.log



vpath %.tex ${CONTENT_DIRS} ${FIG_DIRS} ${TABLE_DIRS} ${CODE_DIRS} ${BASE_SRC}


.PHONY: edit view clean all

all: ${DOCUMENT}

${DIRS}: %:
	@mkdir -p $@

${DOCUMENT}: ${SOURCES} | ${BUILD} ${DOC} ${LOG}
	@find ${BUILD} -maxdepth 1 -type f  -exec rm {} \;
	@rm -f ${LOG}/run?.log ${LOG}/bib.log
	@${LATEX_CMD1}
	@${BIBTEX_CMD}
	@${LATEX_CMD2}
	@${LATEX_CMD3}
	@mv ${TARGET_PDF} $@

clean:
	@echo "(CLEAN) ${GARBAGE}"
	@rm -rf ${GARBAGE}

edit:
	@${EDITOR} ${SOURCES} ${BIBFILES}

view: ${DOCUMENT}
	@${VIEWER} $< &

view_log:
	@${EDITOR} $(wildcard ${LOG}/*.log)
